From: Wang Yugui <wangyugui@e16-tech.com>
To: linux-btrfs@vger.kernel.org
Cc: Wang Yugui <wangyugui@e16-tech.com>
Subject: [PATCH] btrfs: round down stripe size and chunk size to pow of 2
Date: Tue, 16 Aug 2022 09:46:03 +0800 [thread overview]
Message-ID: <20220816014603.1247-1-wangyugui@e16-tech.com> (raw)
In decide_stripe_size_regular(), when new disk is added to RAID0/RAID10/RAID56,
it is better to alloc/reuse the free space if stripe size is kept or
changed to 1/2. so stripe size and chunk size of pow of 2 will be more
friendly.
Signed-off-by: Wang Yugui <wangyugui@e16-tech.com>
---
fs/btrfs/volumes.c | 20 +++++++++-----------
1 file changed, 9 insertions(+), 11 deletions(-)
diff --git a/fs/btrfs/volumes.c b/fs/btrfs/volumes.c
index 6595755..fab9765 100644
--- a/fs/btrfs/volumes.c
+++ b/fs/btrfs/volumes.c
@@ -5083,9 +5083,9 @@ static void init_alloc_chunk_ctl_policy_regular(
if (ctl->type & BTRFS_BLOCK_GROUP_SYSTEM)
ctl->devs_max = min_t(int, ctl->devs_max, BTRFS_MAX_DEVS_SYS_CHUNK);
- /* We don't want a chunk larger than 10% of writable space */
- ctl->max_chunk_size = min(div_factor(fs_devices->total_rw_bytes, 1),
- ctl->max_chunk_size);
+ /* We don't want a chunk larger than 1/8 of writable space */
+ ctl->max_chunk_size = min_t(u64, ctl->max_chunk_size,
+ rounddown_pow_of_two(fs_devices->total_rw_bytes >> 3));
ctl->dev_extent_min = BTRFS_STRIPE_LEN * ctl->dev_stripes;
}
@@ -5143,10 +5143,9 @@ static void init_alloc_chunk_ctl_policy_zoned(
BUG();
}
- /* We don't want a chunk larger than 10% of writable space */
- limit = max(round_down(div_factor(fs_devices->total_rw_bytes, 1),
- zone_size),
- min_chunk_size);
+ /* We don't want a chunk larger than 1/8 of writable space */
+ limit = max_t(u64, min_chunk_size,
+ rounddown_pow_of_two(fs_devices->total_rw_bytes >> 3));
ctl->max_chunk_size = min(limit, ctl->max_chunk_size);
ctl->dev_extent_min = zone_size * ctl->dev_stripes;
}
@@ -5284,13 +5283,12 @@ static int decide_stripe_size_regular(struct alloc_chunk_ctl *ctl,
*/
if (ctl->stripe_size * data_stripes > ctl->max_chunk_size) {
/*
* Reduce stripe_size, round it up to a 16MB boundary again and
* then use it, unless it ends up being even bigger than the
* previous value we had already.
*/
- ctl->stripe_size = min(round_up(div_u64(ctl->max_chunk_size,
- data_stripes), SZ_16M),
- ctl->stripe_size);
+ ctl->stripe_size = min_t(u64, ctl->stripe_size,
+ rounddown_pow_of_two(div_u64(ctl->max_chunk_size, data_stripes)));
}
/* Align to BTRFS_STRIPE_LEN */
@@ -5284,13 +5283,12 @@ static int decide_stripe_size_regular(struct alloc_chunk_ctl *ctl,
if (ctl->stripe_size * data_stripes > ctl->max_chunk_size) {
/*
- * Reduce stripe_size, round it up to a 16MB boundary again and
+ * Reduce stripe_size, round down to pow of 2 boundary again and
* then use it, unless it ends up being even bigger than the
* previous value we had already.
--
2.36.2
next reply other threads:[~2022-08-16 6:46 UTC|newest]
Thread overview: 3+ messages / expand[flat|nested] mbox.gz Atom feed top
2022-08-16 1:46 Wang Yugui [this message]
2022-08-16 7:54 ` [PATCH] btrfs: round down stripe size and chunk size to pow of 2 Qu Wenruo
2022-08-16 13:29 ` Wang Yugui
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20220816014603.1247-1-wangyugui@e16-tech.com \
--to=wangyugui@e16-tech.com \
--cc=linux-btrfs@vger.kernel.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox